@@ -3,7 +3,10 @@
 #include <R.h>
 #include <R_ext/Applic.h>
 
-#define MI(i, j, nrows) ( (int) ((j)*(nrows) + (i)) ) /* index to treat a vector as a matrix. ith row, jth column. Fills columns first, as in R */
+ /* index to treat a vector as a matrix. ith row, jth column. Fills columns first, as in R */
+#define MI(i, j, nrows) ( (int) ((j)*(nrows) + (i)) )
+/* index to treat a vector as a 3-dimensional array. Left-most index varies fastest, as in R */
+#define MI3(i, j, k, n1, n2) ( (int) ((k)*(n1*n2) + (j)*(n1) + (i)) )
 
 /* Macros to switch quickly between C and S memory handling. Currently not used */
 
@@ -18,6 +21,7 @@
 #define MSM_FREE(var) 
 #endif 
 
+typedef double * Array3;
 typedef double * Matrix;
 typedef int * iMatrix;
 typedef double * vector;
@@ -49,12 +53,10 @@ struct qmodel {
     int nst;
     int npars;
     int *ivector;
-    int *constr;
     double *intens;
 };
 
 struct qcmodel {
-    int *constr;
     int *ncovs;
     double *coveffect;
 };
@@ -86,10 +88,15 @@ typedef struct cmodel cmodel;
 typedef struct hmodel hmodel;
 
 double qij(int i, int j, vector intens, ivector qvector, int nstates);
+double pijdeath(int r, int s, Matrix pmat, vector intens, ivector qvector, int n);
 void Pmat(Matrix pmat, double t, vector intens, int *qvector, int nstates, int exacttimes, int debug);
+void DPmat(Array3 dpmat, double t, vector intens, ivector qvector, int n, int np, int exacttimes, int debug);
+void dpijdeath(int r, int s, Array3 dpmat, Matrix pmat, vector intens, ivector qvector, int n, int np, vector dcontrib);
 int repeated_entries(vector vec, int n);
 
 double logit(double x);
 double expit(double x);
 double identity(double x);
 int all_equal(double x, double y);
+
+void MatrixExpPadeR(double *ExpAt, double *A, int *n, double *t);
